@@ -34,9 +34,7 @@ import java.security.cert.X509Certificate;
 import java.text.SimpleDateFormat;
 import java.util.Arrays;
 import java.util.Collection;
-import java.util.Date;
 import java.util.HashSet;
-import java.util.Set;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSDictionary;
@@ -70,6 +68,7 @@ import org.bouncycastle.cms.jcajce.JcaSimpleSignerInfoVerifierBuilder;
 import org.bouncycastle.operator.OperatorCreationException;
 import org.bouncycastle.tsp.TSPException;
 import org.bouncycastle.tsp.TimeStampToken;
+import org.bouncycastle.tsp.TimeStampTokenInfo;
 import org.bouncycastle.util.CollectionStore;
 import org.bouncycastle.util.Selector;
 import org.bouncycastle.util.Store;
@@ -136,7 +135,7 @@ public final class ShowSignature
                 for (PDSignature sig : document.getSignatureDictionaries())
                 {
                     COSDictionary sigDict = sig.getCOSObject();
-                    COSString contents = (COSString) sigDict.getDictionaryObject(COSName.CONTENTS);
+                    byte[] contents = sig.getContents();
 
                     // download the signed content
                     FileInputStream fis = new FileInputStream(infile);
@@ -171,9 +170,8 @@ public final class ShowSignature
                         else if (subFilter.equals("adbe.pkcs7.sha1"))
                         {
                             // example: PDFBOX-1452.pdf
-                            byte[] certData = contents.getBytes();
                             CertificateFactory factory = CertificateFactory.getInstance("X.509");
-                            ByteArrayInputStream certStream = new ByteArrayInputStream(certData);
+                            ByteArrayInputStream certStream = new ByteArrayInputStream(contents);
                             Collection<? extends Certificate> certs = factory.generateCertificates(certStream);
                             System.out.println("certs=" + certs);
 
@@ -233,7 +231,7 @@ public final class ShowSignature
                                 {
                                     @SuppressWarnings("unchecked")
                                     Store<X509CertificateHolder> store = new JcaCertStore(certs);
-                                    verifyCertificateChain(store, cert, sig.getSignDate().getTime());
+                                    SigUtils.verifyCertificateChain(store, cert, sig.getSignDate().getTime());
                                 }
                             }
                         }
@@ -262,7 +260,7 @@ public final class ShowSignature
                         long fileLen = infile.length();
                         long rangeMax = byteRange[2] + (long) byteRange[3];
                         // multiply content length with 2 (because it is in hex in the PDF) and add 2 for < and >
-                        int contentLen = contents.getString().length() * 2 + 2;
+                        int contentLen = contents.length * 2 + 2;
                         if (fileLen != rangeMax || byteRange[0] != 0 || byteRange[1] + contentLen != byteRange[2])
                         {
                             // a false result doesn't necessarily mean that the PDF is a fake
@@ -298,7 +296,7 @@ public final class ShowSignature
         }
     }
 
-    private void checkContentValueWithFile(File file, int[] byteRange, COSString contents) throws IOException
+    private void checkContentValueWithFile(File file, int[] byteRange, byte[] contents) throws IOException
     {
         // https://stackoverflow.com/questions/55049270
         // comment by mkl: check whether gap contains a hex value equal
@@ -312,7 +310,7 @@ public final class ShowSignature
             System.err.println("'<' expected at offset " + byteRange[1] + ", but got " + (char) c);
         }
         byte[] contentFromFile = raf.readFully(byteRange[2] - byteRange[1] - 2);
-        byte[] contentAsHex = Hex.getString(contents.getBytes()).getBytes(Charsets.US_ASCII);
+        byte[] contentAsHex = Hex.getString(contents).getBytes(Charsets.US_ASCII);
         if (contentFromFile.length != contentAsHex.length)
         {
             System.err.println("Raw content length from file is " +
@@ -350,23 +348,27 @@ public final class ShowSignature
         raf.close();
     }
 
-    private void verifyETSIdotRFC3161(byte[] buf, COSString contents)
+    private void verifyETSIdotRFC3161(byte[] buf, byte[] contents)
             throws CertificateException, CMSException, IOException, OperatorCreationException,
             TSPException, NoSuchAlgorithmException, CertificateVerificationException
     {
-        TimeStampToken timeStampToken = new TimeStampToken(new CMSSignedData(contents.getBytes()));
-        System.out.println("Time stamp gen time: " + timeStampToken.getTimeStampInfo().getGenTime());
-        System.out.println("Time stamp tsa name: " + timeStampToken.getTimeStampInfo().getTsa().getName());
+        TimeStampToken timeStampToken = new TimeStampToken(new CMSSignedData(contents));
+        TimeStampTokenInfo timeStampInfo = timeStampToken.getTimeStampInfo();
+        System.out.println("Time stamp gen time: " + timeStampInfo.getGenTime());
+        if (timeStampInfo.getTsa() != null)
+        {
+            System.out.println("Time stamp tsa name: " + timeStampInfo.getTsa().getName());
+        }
         
         CertificateFactory factory = CertificateFactory.getInstance("X.509");
-        ByteArrayInputStream certStream = new ByteArrayInputStream(contents.getBytes());
+        ByteArrayInputStream certStream = new ByteArrayInputStream(contents);
         Collection<? extends Certificate> certs = factory.generateCertificates(certStream);
         System.out.println("certs=" + certs);
         
-        String hashAlgorithm = timeStampToken.getTimeStampInfo().getMessageImprintAlgOID().getId();
+        String hashAlgorithm = timeStampInfo.getMessageImprintAlgOID().getId();
         // compare the hash of the signed content with the hash in the timestamp
         if (Arrays.equals(MessageDigest.getInstance(hashAlgorithm).digest(buf),
-                timeStampToken.getTimeStampInfo().getMessageImprintDigest()))
+                timeStampInfo.getMessageImprintDigest()))
         {
             System.out.println("ETSI.RFC3161 timestamp signature verified");
         }
@@ -378,9 +380,9 @@ public final class ShowSignature
         X509Certificate certFromTimeStamp = (X509Certificate) certs.iterator().next();
         SigUtils.checkTimeStampCertificateUsage(certFromTimeStamp);
         SigUtils.validateTimestampToken(timeStampToken);
-                verifyCertificateChain(timeStampToken.getCertificates(),
+                SigUtils.verifyCertificateChain(timeStampToken.getCertificates(),
                 certFromTimeStamp,
-                timeStampToken.getTimeStampInfo().getGenTime());
+                timeStampInfo.getGenTime());
     }
 
     /**
@@ -395,7 +397,7 @@ public final class ShowSignature
      * @throws GeneralSecurityException
      * @throws TSPException
      */
-    private void verifyPKCS7(byte[] byteArray, COSString contents, PDSignature sig)
+    private void verifyPKCS7(byte[] byteArray, byte[] contents, PDSignature sig)
             throws CMSException, OperatorCreationException,
                    IOException, GeneralSecurityException, TSPException, CertificateVerificationException
     {
@@ -403,7 +405,7 @@ public final class ShowSignature
         // http://stackoverflow.com/a/26702631/535646
         // http://stackoverflow.com/a/9261365/535646
         CMSProcessable signedContent = new CMSProcessableByteArray(byteArray);
-        CMSSignedData signedData = new CMSSignedData(signedContent, contents.getBytes());
+        CMSSignedData signedData = new CMSSignedData(signedContent, contents);
         @SuppressWarnings("unchecked")
         Store<X509CertificateHolder> certificatesStore = signedData.getCertificates();
         if (certificatesStore.getMatches(null).isEmpty())
@@ -448,10 +450,23 @@ public final class ShowSignature
             HashSet<X509CertificateHolder> certificateHolderSet = new HashSet<X509CertificateHolder>();
             certificateHolderSet.addAll(certificatesStore.getMatches(null));
             certificateHolderSet.addAll(timeStampToken.getCertificates().getMatches(null));
-            verifyCertificateChain(new CollectionStore<X509CertificateHolder>(certificateHolderSet),
+            SigUtils.verifyCertificateChain(new CollectionStore<X509CertificateHolder>(certificateHolderSet),
                     certFromTimeStamp,
                     timeStampToken.getTimeStampInfo().getGenTime());
             SigUtils.checkTimeStampCertificateUsage(certFromTimeStamp);
+
+            // compare the hash of the signature with the hash in the timestamp
+            byte[] tsMessageImprintDigest = timeStampToken.getTimeStampInfo().getMessageImprintDigest();
+            String hashAlgorithm = timeStampToken.getTimeStampInfo().getMessageImprintAlgOID().getId();
+            byte[] sigMessageImprintDigest = MessageDigest.getInstance(hashAlgorithm).digest(signerInformation.getSignature());
+            if (Arrays.equals(tsMessageImprintDigest, sigMessageImprintDigest))
+            {
+                System.out.println("timestamp signature verified");
+            }
+            else
+            {
+                System.err.println("timestamp signature verification failed");
+            }
         }
 
         try
@@ -519,7 +534,7 @@ public final class ShowSignature
 
             if (sig.getSignDate() != null)
             {
-                verifyCertificateChain(certificatesStore, certFromSignedData, sig.getSignDate().getTime());
+                SigUtils.verifyCertificateChain(certificatesStore, certFromSignedData, sig.getSignDate().getTime());
             }
             else
             {
@@ -528,32 +543,6 @@ public final class ShowSignature
         }
     }
 
-    private void verifyCertificateChain(Store<X509CertificateHolder> certificatesStore,
-            X509Certificate certFromSignedData, Date signDate)
-            throws CertificateVerificationException, CertificateException
-    {
-        // Verify certificate chain (new since 11/2018)
-        // Please post bad PDF files that succeed and
-        // good PDF files that fail in
-        // https://issues.apache.org/jira/browse/PDFBOX-3017
-        Collection<X509CertificateHolder> certificateHolders = certificatesStore.getMatches(null);
-        Set<X509Certificate> additionalCerts = new HashSet<X509Certificate>();
-        JcaX509CertificateConverter certificateConverter = new JcaX509CertificateConverter();
-        for (X509CertificateHolder certHolder : certificateHolders)
-        {
-            X509Certificate certificate = certificateConverter.getCertificate(certHolder);
-            if (!certificate.equals(certFromSignedData))
-            {
-                additionalCerts.add(certificate);
-            }
-        }
-        CertificateVerifier.verifyCertificate(certFromSignedData, additionalCerts, true, signDate);
-        //TODO check whether the root certificate is in our trusted list.
-        // For the EU, get a list here:
-        // https://ec.europa.eu/digital-single-market/en/eu-trusted-lists-trust-service-providers
-        // ( getRootCertificates() is not helpful because these are SSL certificates)
-    }
-
     /**
      * Analyzes the DSS-Dictionary (Document Security Store) of the document. Which is used for
      * signature validation. The DSS is defined in PAdES Part 4 - Long Term Validation.
