@@ -1435,23 +1435,16 @@ public class COSDictionary extends COSBase implements COSUpdateInfo
     }
 
     /**
-     * This will add all of the dictionaries keys/values to this dictionary.
-     * Only called when adding keys to a trailer that already exists.
+     * This will add all of the dictionaries keys/values to this dictionary. Existing key/value pairs will be
+     * overwritten.
      *
-     * @param dic The dictionaries to get the keys from.
+     * @param dic The dictionaries to get the key/value pairs from.
      */
     public void addAll(COSDictionary dic)
     {
         for (Map.Entry<COSName, COSBase> entry : dic.entrySet())
         {
-            /*
-             * If we're at a second trailer, we have a linearized pdf file, meaning that the first Size entry represents
-             * all of the objects so we don't need to grab the second.
-             */
-            if (!COSName.SIZE.equals(entry.getKey()) || !items.containsKey(COSName.SIZE))
-            {
-                setItem(entry.getKey(), entry.getValue());
-            }
+            setItem(entry.getKey(), entry.getValue());
         }
     }
 
