@@ -74,10 +74,6 @@ public final class StandardSecurityHandler extends SecurityHandler
     // hashes used for Algorithm 2.B, depending on remainder from E modulo 3
     private static final String[] HASHES_2B = new String[] {"SHA-256", "SHA-384", "SHA-512"};
 
-    private static final int DEFAULT_VERSION = 1;
-
-    private StandardProtectionPolicy policy;
-
     /**
      * Constructor.
      */
@@ -88,37 +84,12 @@ public final class StandardSecurityHandler extends SecurityHandler
     /**
      * Constructor used for encryption.
      *
-     * @param p The protection policy.
+     * @param standardProtectionPolicy The protection policy.
      */
-    public StandardSecurityHandler(StandardProtectionPolicy p)
+    public StandardSecurityHandler(StandardProtectionPolicy standardProtectionPolicy)
     {
-        policy = p;
-        keyLength = policy.getEncryptionKeyLength();
-    }
-
-    /**
-     * Computes the version number of the StandardSecurityHandler
-     * based on the encryption key length.
-     * See PDF Spec 1.6 p 93 and PDF 1.7 AEL3
-     *
-     * @return The computed version number.
-     */
-    private int computeVersionNumber()
-    {
-        if(keyLength == 40)
-        {
-            return DEFAULT_VERSION;
-        }
-        else if (keyLength == 128 && policy.isPreferAES())
-        {
-            return 4;
-        }
-        else if (keyLength == 256)
-        {
-            return 5;
-        }
-
-        return 2;
+        setProtectionPolicy(standardProtectionPolicy);
+        setKeyLength(standardProtectionPolicy.getEncryptionKeyLength());
     }
 
     /**
@@ -132,7 +103,9 @@ public final class StandardSecurityHandler extends SecurityHandler
      */
     private int computeRevisionNumber(int version)
     {
-        if(version < 2 && !policy.getPermissions().hasAnyRevision3PermissionSet())
+        StandardProtectionPolicy protectionPolicy = (StandardProtectionPolicy) getProtectionPolicy();
+        AccessPermission permissions = protectionPolicy.getPermissions();
+        if (version < 2 && !permissions.hasAnyRevision3PermissionSet())
         {
             return 2;
         }
@@ -145,7 +118,7 @@ public final class StandardSecurityHandler extends SecurityHandler
         {
             return 4;
         }
-        if ( version == 2 || version == 3 || policy.getPermissions().hasAnyRevision3PermissionSet())
+        if (version == 2 || version == 3 || permissions.hasAnyRevision3PermissionSet())
         {
             return 3;
         }
@@ -234,7 +207,7 @@ public final class StandardSecurityHandler extends SecurityHandler
                         ownerKey, dicRevision, dicLength );
             }
             
-            encryptionKey =
+            setEncryptionKey(
                 computeEncryptedKey(
                     computedPassword,
                     ownerKey, userKey, oe, ue,
@@ -242,7 +215,7 @@ public final class StandardSecurityHandler extends SecurityHandler
                     documentIDBytes,
                     dicRevision,
                     dicLength,
-                    encryptMetadata, true );
+                    encryptMetadata, true));
         }
         else if( isUserPassword(password.getBytes(passwordCharset), userKey, ownerKey,
                            dicPermissions, documentIDBytes, dicRevision,
@@ -252,7 +225,7 @@ public final class StandardSecurityHandler extends SecurityHandler
             currentAccessPermission.setReadOnly();
             setCurrentAccessPermission(currentAccessPermission);
             
-            encryptionKey =
+            setEncryptionKey(
                 computeEncryptedKey(
                     password.getBytes(passwordCharset),
                     ownerKey, userKey, oe, ue,
@@ -260,7 +233,7 @@ public final class StandardSecurityHandler extends SecurityHandler
                     documentIDBytes,
                     dicRevision,
                     dicLength,
-                    encryptMetadata, false );
+                    encryptMetadata, false));
         }
         else
         {
@@ -314,7 +287,7 @@ public final class StandardSecurityHandler extends SecurityHandler
             // "Decrypt the 16-byte Perms string using AES-256 in ECB mode with an 
             // initialization vector of zero and the file encryption key as the key."
             Cipher cipher = Cipher.getInstance("AES/ECB/NoPadding");
-            cipher.init(Cipher.DECRYPT_MODE, new SecretKeySpec(encryptionKey, "AES"));
+            cipher.init(Cipher.DECRYPT_MODE, new SecretKeySpec(getEncryptionKey(), "AES"));
             byte[] perms = cipher.doFinal(encryption.getPerms());
             
             // "Verify that bytes 9-11 of the result are the characters ‘a’, ‘d’, ‘b’."
@@ -371,10 +344,11 @@ public final class StandardSecurityHandler extends SecurityHandler
             encryptionDictionary.removeV45filters();
         }
         encryptionDictionary.setRevision(revision);
-        encryptionDictionary.setLength(keyLength);
+        encryptionDictionary.setLength(getKeyLength());
 
-        String ownerPassword = policy.getOwnerPassword();
-        String userPassword = policy.getUserPassword();
+        StandardProtectionPolicy protectionPolicy = (StandardProtectionPolicy) getProtectionPolicy();
+        String ownerPassword = protectionPolicy.getOwnerPassword();
+        String userPassword = protectionPolicy.getUserPassword();
         if( ownerPassword == null )
         {
             ownerPassword = "";
@@ -390,11 +364,11 @@ public final class StandardSecurityHandler extends SecurityHandler
             ownerPassword = userPassword;
         }
 
-        int permissionInt = policy.getPermissions().getPermissionBytes();
+        int permissionInt = protectionPolicy.getPermissions().getPermissionBytes();
 
         encryptionDictionary.setPermissions(permissionInt);
 
-        int length = keyLength/8;
+        int length = getKeyLength()/8;
 
         if (revision == 6)
         {
@@ -423,8 +397,8 @@ public final class StandardSecurityHandler extends SecurityHandler
             Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
 
             // make a random 256-bit file encryption key
-            encryptionKey = new byte[32];
-            rnd.nextBytes(encryptionKey);
+            setEncryptionKey(new byte[32]);
+            rnd.nextBytes(getEncryptionKey());
 
             // Algorithm 8a: Compute U
             byte[] userPasswordBytes = truncate127(userPassword.getBytes(Charsets.UTF_8));
@@ -441,7 +415,7 @@ public final class StandardSecurityHandler extends SecurityHandler
                     userPasswordBytes, null);
             cipher.init(Cipher.ENCRYPT_MODE, new SecretKeySpec(hashUE, "AES"),
                     new IvParameterSpec(new byte[16]));
-            byte[] ue = cipher.doFinal(encryptionKey);
+            byte[] ue = cipher.doFinal(getEncryptionKey());
 
             // Algorithm 9a: Compute O
             byte[] ownerPasswordBytes = truncate127(ownerPassword.getBytes(Charsets.UTF_8));
@@ -458,7 +432,7 @@ public final class StandardSecurityHandler extends SecurityHandler
                     ownerPasswordBytes, u);
             cipher.init(Cipher.ENCRYPT_MODE, new SecretKeySpec(hashOE, "AES"),
                     new IvParameterSpec(new byte[16]));
-            byte[] oe = cipher.doFinal(encryptionKey);
+            byte[] oe = cipher.doFinal(getEncryptionKey());
 
             // Set keys and other required constants in encryption dictionary
             encryptionDictionary.setUserKey(u);
@@ -487,7 +461,7 @@ public final class StandardSecurityHandler extends SecurityHandler
                 perms[i] = (byte) rnd.nextInt();
             }
 
-            cipher.init(Cipher.ENCRYPT_MODE, new SecretKeySpec(encryptionKey, "AES"),
+            cipher.init(Cipher.ENCRYPT_MODE, new SecretKeySpec(getEncryptionKey(), "AES"),
                     new IvParameterSpec(new byte[16]));
 
             byte[] permsEnc = cipher.doFinal(perms);
@@ -537,8 +511,8 @@ public final class StandardSecurityHandler extends SecurityHandler
                 userPassword.getBytes(Charsets.ISO_8859_1),
                 ownerBytes, permissionInt, id.getBytes(), revision, length, true);
 
-        encryptionKey = computeEncryptedKey(userPassword.getBytes(Charsets.ISO_8859_1), ownerBytes,
-                null, null, null, permissionInt, id.getBytes(), revision, length, true, false);
+        setEncryptionKey(computeEncryptedKey(userPassword.getBytes(Charsets.ISO_8859_1), ownerBytes,
+                null, null, null, permissionInt, id.getBytes(), revision, length, true, false));
 
         encryptionDictionary.setOwnerKey(ownerBytes);
         encryptionDictionary.setUserKey(userBytes);
@@ -553,7 +527,7 @@ public final class StandardSecurityHandler extends SecurityHandler
     {
         PDCryptFilterDictionary cryptFilterDictionary = new PDCryptFilterDictionary();
         cryptFilterDictionary.setCryptFilterMethod(aesVName);
-        cryptFilterDictionary.setLength(keyLength);
+        cryptFilterDictionary.setLength(getKeyLength());
         encryptionDictionary.setStdCryptFilterDictionary(cryptFilterDictionary);
         encryptionDictionary.setStreamFilterName(COSName.STD_CF);
         encryptionDictionary.setStringFilterName(COSName.STD_CF);
@@ -1084,7 +1058,7 @@ public final class StandardSecurityHandler extends SecurityHandler
     {
         try
         {
-            MessageDigest md = MessageDigest.getInstance("SHA-256");
+            MessageDigest md = MessageDigests.getSHA256();
             byte[] k = md.digest(input);
             
             byte[] e = null;
@@ -1153,22 +1127,14 @@ public final class StandardSecurityHandler extends SecurityHandler
         }
     }
 
-    private static byte[] computeSHA256(byte[] input, byte[] password, byte[] userKey) 
-            throws IOException
+    private static byte[] computeSHA256(byte[] input, byte[] password, byte[] userKey)
     {
-        try
-        {
-            MessageDigest md = MessageDigest.getInstance("SHA-256");
-            md.update(input);
-            md.update(password);
-            return userKey == null ? md.digest() : md.digest(userKey);
-        }
-        catch (NoSuchAlgorithmException e)
-        {
-            throw new IOException(e);
-        }
+        MessageDigest md = MessageDigests.getSHA256();
+        md.update(input);
+        md.update(password);
+        return userKey == null ? md.digest() : md.digest(userKey);
     }
-    
+
     private static byte[] concat(byte[] a, byte[] b)
     {
         byte[] o = new byte[a.length + b.length];
@@ -1210,13 +1176,4 @@ public final class StandardSecurityHandler extends SecurityHandler
         {
         }
     }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public boolean hasProtectionPolicy()
-    {
-        return policy != null;
-    }
 }
