@@ -25,6 +25,7 @@ import java.awt.color.ICC_ColorSpace;
 import java.awt.color.ICC_Profile;
 import java.awt.image.BufferedImage;
 import java.awt.image.WritableRaster;
+import java.io.BufferedInputStream;
 import java.io.IOException;
 
 import java.io.InputStream;
@@ -96,11 +97,7 @@ public class PDDeviceCMYK extends PDDeviceColorSpace
 
         String name = "/org/apache/pdfbox/resources/icc/ISOcoated_v2_300_bas.icc";
 
-        InputStream is = PDDeviceCMYK.class.getResourceAsStream(name);
-        if (is == null)
-        {
-            throw new IOException("Error loading resource: " + name);
-        }
+        InputStream is = new BufferedInputStream(PDDeviceCMYK.class.getResourceAsStream(name));
         ICC_Profile iccProfile = ICC_Profile.getInstance(is);
         is.close();
 
@@ -138,6 +135,14 @@ public class PDDeviceCMYK extends PDDeviceColorSpace
         return awtColorSpace.toRGB(value);
     }
 
+    @Override
+    public BufferedImage toRawImage(WritableRaster raster) throws IOException
+    {
+        // Device CMYK is not specified, as its the colors of whatever device you use.
+        // The user should fallback to the RGB image
+        return null;
+    }
+
     @Override
     public BufferedImage toRGBImage(WritableRaster raster) throws IOException
     {
