@@ -82,6 +82,7 @@ import org.apache.pdfbox.pdmodel.graphics.color.PDColorSpace;
 import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceGray;
 import org.apache.pdfbox.pdmodel.graphics.color.PDICCBased;
 import org.apache.pdfbox.pdmodel.graphics.color.PDPattern;
+import org.apache.pdfbox.pdmodel.graphics.color.PDSeparation;
 import org.apache.pdfbox.pdmodel.graphics.form.PDFormXObject;
 import org.apache.pdfbox.pdmodel.graphics.form.PDTransparencyGroup;
 import org.apache.pdfbox.pdmodel.graphics.image.PDImage;
@@ -162,6 +163,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
 
     private final RenderDestination destination;
     private final RenderingHints renderingHints;
+    private final float imageDownscalingOptimizationThreshold;
 
     static final int JAVA_VERSION = PageDrawer.getJavaVersion();
 
@@ -190,6 +192,8 @@ public class PageDrawer extends PDFGraphicsStreamEngine
         this.subsamplingAllowed = parameters.isSubsamplingAllowed();
         this.destination = parameters.getDestination();
         this.renderingHints = parameters.getRenderingHints();
+        this.imageDownscalingOptimizationThreshold =
+                parameters.getImageDownscalingOptimizationThreshold();
     }
 
     /**
@@ -212,7 +216,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
     public void setAnnotationFilter(AnnotationFilter annotationFilter)
     {
         this.annotationFilter = annotationFilter;
-    }    
+    }
     
     /**
      * Returns the parent renderer.
@@ -332,7 +336,14 @@ public class PageDrawer extends PDFGraphicsStreamEngine
     protected Paint getPaint(PDColor color) throws IOException
     {
         PDColorSpace colorSpace = color.getColorSpace();
-        if (!(colorSpace instanceof PDPattern))
+        if (colorSpace instanceof PDSeparation &&
+                "None".equals(((PDSeparation) colorSpace).getColorantName()))
+        {
+            // PDFBOX-4900: "The special colorant name None shall not produce any visible output"
+            //TODO better solution needs to be found for all occurences where toRGB is called
+            return new Color(0, 0, 0, 0);
+        }
+        else if (!(colorSpace instanceof PDPattern))
         {
             float[] rgb = colorSpace.toRGB(color.getComponents());
             return new Color(clampColor(rgb[0]), clampColor(rgb[1]), clampColor(rgb[2]));
@@ -368,7 +379,6 @@ public class PageDrawer extends PDFGraphicsStreamEngine
                 }
                 return shading.toPaint(Matrix.concatenate(getInitialMatrix(),
                                                           shadingPattern.getMatrix()));
-
             }
         }
     }
@@ -382,7 +392,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
         {
             if (clippingPath.getPathIterator(null).isDone())
             {
-                // PDFBOX-4822: avoid bug with java printing that empty clipping path is ignored by
+                // PDFBOX-4821: avoid bug with java printing that empty clipping path is ignored by
                 // replacing with empty rectangle, works because this is not an empty path
                 graphics.setClip(new Rectangle());
             }
@@ -758,8 +768,14 @@ public class PageDrawer extends PDFGraphicsStreamEngine
         }
         int lineCap = Math.min(2, Math.max(0, state.getLineCap()));
         int lineJoin = Math.min(2, Math.max(0, state.getLineJoin()));
+        float miterLimit = state.getMiterLimit();
+        if (miterLimit < 1)
+        {
+            LOG.warn("Miter limit must be >= 1, value " + miterLimit + " is ignored");
+            miterLimit = 10;
+        }
         return new BasicStroke(lineWidth, lineCap, lineJoin,
-                               state.getMiterLimit(), dashArray, phaseStart);
+                               miterLimit, dashArray, phaseStart);
     }
 
     private float[] getDashArray(PDLineDashPattern dashPattern)
@@ -1003,7 +1019,12 @@ public class PageDrawer extends PDFGraphicsStreamEngine
         if (clipWindingRule != -1)
         {
             linePath.setWindingRule(clipWindingRule);
-            getGraphicsState().intersectClippingPath(linePath);
+
+            if (!linePath.getPathIterator(null).isDone())
+            {
+                // PDFBOX-4949 / PDF.js 12306: don't clip if "W n" only
+                getGraphicsState().intersectClippingPath(linePath);
+            }
 
             // PDFBOX-3836: lastClip needs to be reset, because after intersection it is still the same 
             // object, thus setClip() would believe that it is cached.
@@ -1031,11 +1052,13 @@ public class PageDrawer extends PDFGraphicsStreamEngine
 
         if (!pdImage.getInterpolate())
         {
-            boolean isScaledUp = pdImage.getWidth() < Math.round(at.getScaleX()) ||
-                                 pdImage.getHeight() < Math.round(at.getScaleY());
-
             // if the image is scaled down, we use smooth interpolation, eg PDFBOX-2364
             // only when scaled up do we use nearest neighbour, eg PDFBOX-2302 / mori-cvpr01.pdf
+            // PDFBOX-4930: we use the sizes of the ARGB image. These can be different
+            // than the original sizes of the base image, when the mask is bigger.
+            boolean isScaledUp = pdImage.getImage().getWidth() < Math.round(at.getScaleX()) ||
+                                 pdImage.getImage().getHeight() < Math.round(at.getScaleY());
+
             if (isScaledUp)
             {
                 graphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
@@ -1080,6 +1103,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
                 Graphics2D g = (Graphics2D) renderedPaint.getGraphics();
                 g.translate(-bounds.getMinX(), -bounds.getMinY());
                 g.setPaint(paint);
+                g.setRenderingHints(graphics.getRenderingHints());
                 g.fill(bounds);
                 g.dispose();
 
@@ -1091,6 +1115,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
                 AffineTransform imageTransform = new AffineTransform(at);
                 imageTransform.scale(1.0 / mask.getWidth(), -1.0 / mask.getHeight());
                 imageTransform.translate(0, -mask.getHeight());
+                g.setRenderingHints(graphics.getRenderingHints());
                 g.drawImage(mask, imageTransform, null);
                 g.dispose();
 
@@ -1190,18 +1215,23 @@ public class PageDrawer extends PDFGraphicsStreamEngine
         graphics.setComposite(getGraphicsState().getNonStrokingJavaComposite());
         setClip();
         AffineTransform imageTransform = new AffineTransform(at);
+        int width = image.getWidth();
+        int height = image.getHeight();
+        imageTransform.scale(1.0 / width, -1.0 / height);
+        imageTransform.translate(0, -height);
+
         PDSoftMask softMask = getGraphicsState().getSoftMask();
         if( softMask != null )
         {
-            imageTransform.scale(1, -1);
-            imageTransform.translate(0, -1);
-            Paint awtPaint = new TexturePaint(image,
-                    new Rectangle2D.Double(imageTransform.getTranslateX(), imageTransform.getTranslateY(),
-                            imageTransform.getScaleX(), imageTransform.getScaleY()));
+            Rectangle2D rectangle = new Rectangle2D.Float(0, 0, width, height);
+            Paint awtPaint = new TexturePaint(image, rectangle);
             awtPaint = applySoftMaskToPaint(awtPaint, softMask);
             graphics.setPaint(awtPaint);
-            Rectangle2D unitRect = new Rectangle2D.Float(0, 0, 1, 1);
-            graphics.fill(at.createTransformedShape(unitRect));
+
+            AffineTransform originalTransform = graphics.getTransform();
+            graphics.transform(imageTransform);
+            graphics.fill(rectangle);
+            graphics.setTransform(originalTransform);
         }
         else
         {
@@ -1211,12 +1241,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
                 image = applyTransferFunction(image, transfer);
             }
 
-            int width = image.getWidth();
-            int height = image.getHeight();
-            imageTransform.scale(1.0 / width, -1.0 / height);
-            imageTransform.translate(0, -height);
-
-            // PDFBOX-4516, PDFBOX-4527, PDFBOX-4815:
+            // PDFBOX-4516, PDFBOX-4527, PDFBOX-4815, PDFBOX-4886, PDFBOX-4863:
             // graphics.drawImage() has terrible quality when scaling down, even when
             // RenderingHints.VALUE_INTERPOLATION_BICUBIC, VALUE_ALPHA_INTERPOLATION_QUALITY,
             // VALUE_COLOR_RENDER_QUALITY and VALUE_RENDER_QUALITY are all set.
@@ -1225,41 +1250,37 @@ public class PageDrawer extends PDFGraphicsStreamEngine
             // (partly because the method needs integer parameters), only smaller scalings
             // will trigger the workaround. Because of the slowness we only do it if the user
             // expects quality rendering and interpolation.
-            Matrix m = new Matrix(imageTransform);
-            float scaleX = Math.abs(m.getScalingFactorX());
-            float scaleY = Math.abs(m.getScalingFactorY());
-            Image imageToDraw = image;
+            Matrix imageTransformMatrix = new Matrix(imageTransform);
+            AffineTransform graphicsTransformA = graphics.getTransform();
+            Matrix graphicsTransformMatrix = new Matrix(graphicsTransformA);    
+            float scaleX = Math.abs(imageTransformMatrix.getScalingFactorX() * graphicsTransformMatrix.getScalingFactorX());
+            float scaleY = Math.abs(imageTransformMatrix.getScalingFactorY() * graphicsTransformMatrix.getScalingFactorY());
 
-            if ((scaleX < 0.25f || scaleY < 0.25f) &&
+            if ((scaleX < imageDownscalingOptimizationThreshold || scaleY < imageDownscalingOptimizationThreshold) &&
                 RenderingHints.VALUE_RENDER_QUALITY.equals(graphics.getRenderingHint(RenderingHints.KEY_RENDERING)) &&
                 RenderingHints.VALUE_INTERPOLATION_BICUBIC.equals(graphics.getRenderingHint(RenderingHints.KEY_INTERPOLATION)))
             {
-                // PDFBOX-4516, PDFBOX-4527, PDFBOX-4815:
-                // graphics.drawImage() has terrible quality when scaling down, even when
-                // RenderingHints.VALUE_INTERPOLATION_BICUBIC, VALUE_ALPHA_INTERPOLATION_QUALITY,
-                // VALUE_COLOR_RENDER_QUALITY and VALUE_RENDER_QUALITY are all set.
-                // A workaround is to get a pre-scaled image with Image.getScaledInstance()
-                // and then draw that one. To reduce differences in testing
-                // (partly because the method needs integer parameters), only smaller scalings
-                // will trigger the workaround. Because of the slowness we only do it if the user
-                // expects quality rendering and interpolation.
                 int w = Math.round(image.getWidth() * scaleX);
                 int h = Math.round(image.getHeight() * scaleY);
-                if (w < 1)
+                if (w < 1 || h < 1)
                 {
-                    w = 1;
+                    graphics.drawImage(image, imageTransform, null);
+                    return;
                 }
-                if (h < 1)
-                {
-                    h = 1;
-                }
-                imageToDraw = image.getScaledInstance(
-                        w,
-                        h,
-                        Image.SCALE_SMOOTH);
-                imageTransform.scale(1 / scaleX, 1 / scaleY); // remove the scale
+                Image imageToDraw = image.getScaledInstance(w, h, Image.SCALE_SMOOTH);
+                // remove the scale (extracted from w and h, to have it from the rounded values
+                // hoping to reverse the rounding: without this, we get an horizontal line
+                // when rendering PDFJS-8860-Pattern-Size1.pdf at 100% )
+                imageTransform.scale(1f / w * image.getWidth(), 1f / h * image.getHeight());
+                imageTransform.preConcatenate(graphicsTransformA);
+                graphics.setTransform(new AffineTransform());
+                graphics.drawImage(imageToDraw, imageTransform, null);
+                graphics.setTransform(graphicsTransformA);
+            }
+            else
+            {
+                graphics.drawImage(image, imageTransform, null);
             }
-            graphics.drawImage(imageToDraw, imageTransform, null);
         }
     }
 
